/* Copyright (C) 2008 DJ Delorie, see COPYING.DJ for details */

#ifndef __libsupp_h_
#define __libsupp_h_

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/version.h>  /*  gcc 4.3.0 no longer includes this by default.  */
#include <stdarg.h>
#include <stdio.h>


/*
 *  Functions implemented in libsupp 1.0.
 */

#if __DJGPP_MINOR__ < 4
__extension__ typedef signed long long intmax_t;
__extension__ typedef unsigned long long uintmax_t;
__DJ_off_t
# undef  __DJ_off_t
# define __DJ_off_t
#else
# ifndef _OFF_T
__DJ_off_t
#  define _OFF_T
# endif
#endif


int            libsupp__doprnt(const char *_fmt, va_list _ap, FILE *_f);
int            libsupp_asnprintf(char **_sp, size_t _n, const char *_fmt, ...) __attribute__((format (__printf__, 3, 4)));
int            libsupp_asprintf(char **_sp, const char *_fmt, ...) __attribute__((format (__printf__, 2, 3)));
int            libsupp_cprintf(const char *_fmt, ...) __attribute__((format (__printf__, 1, 2)));
int            libsupp_fprintf(FILE *_f, const char *_fmt, ...) __attribute__((format (__printf__, 2, 3)));
int            libsupp_printf(const char *_fmt, ...) __attribute__((format (__printf__, 1, 2)));
int            libsupp_snprintf(char *_s, size_t _n, const char *_fmt, ...) __attribute__((format (__printf__, 3, 4)));
int            libsupp_sprintf(char *_s, const char *_fmt, ...) __attribute__((format (__printf__, 2, 3)));
int            libsupp_vasnprintf(char **_sp, size_t _n, const char *_fmt, va_list _ap) __attribute__((format (__printf__, 3, 0)));
int            libsupp_vasprintf(char **_sp, const char *_fmt, va_list _ap) __attribute__((format (__printf__, 2, 0)));
int            libsupp_vfprintf(FILE *_s, const char *_fmt, va_list _ap) __attribute__((format (__printf__, 2, 0)));
int            libsupp_vprintf(const char *_fmt, va_list _ap) __attribute__((format (__printf__, 1, 0)));
int            libsupp_vsnprintf(char *_s, size_t n, const char *_fmt, va_list _ap) __attribute__((format (__printf__, 3, 0)));
int            libsupp_vsprintf(char *_s, const char *_fmt, va_list _ap) __attribute__((format (__printf__, 2, 0)));


/*
 *  Functions implemented in libsupp 2.0.
 */

int            libsupp_fseeko(FILE *_stream, off_t _offset, int _mode);
off_t          libsupp_ftello(FILE *_stream);

int            libsupp___signbitf(float x) __attribute__((const));
int            libsupp___signbitd(double x) __attribute__((const));
int            libsupp___signbitld(long double x) __attribute__((const));


/*
 *  Functions implemented in libsupp 3.0.
 */

double         libsupp_strtod(const char *_s, char **_sret);
float          libsupp_strtof(const char *_s, char **_sret);
long double    libsupp_strtold(const char *_s, char **_sret);


#include <errno.h>

#ifndef _ERROR_T
typedef int error_t;
# define _ERROR_T
#endif

error_t        libsupp_argz_add(char **_argz, size_t *_argz_len, const char *_str);
error_t        libsupp_argz_add_sep(char **_argz, size_t *_argz_len, const char *_str, int _sep);
error_t        libsupp_argz_append(char **_argz, size_t *_argz_len, const char *_buf, size_t _buf_len);
size_t         libsupp_argz_count(const char *_argz, size_t _argz_len);
error_t        libsupp_argz_create(char *const _argv[], char **_argz, size_t *_argz_len);
error_t        libsupp_argz_create_sep(const char *_str, int _sep, char **_argz, size_t *_argz_len);
void           libsupp_argz_delete(char **_argz, size_t *_argz_len, char *_entry);
void           libsupp_argz_extract(const char *_argz, size_t _argz_len, char **_argv);
error_t        libsupp_argz_insert(char **_argz, size_t *_argz_len, char *_before, const char *_entry);
char          *libsupp_argz_next(const char *_argz, size_t _argz_len, const char *_entry);
error_t        libsupp_argz_replace(char **_argz, size_t *_argz_len, const char *_str, const char *_with, unsigned *_replace_count);
void           libsupp_argz_stringify(char *_argz, size_t _argz_len, int _sep);


/*
 *  Functions implemented in libsupp 4.0.
 */

char          *libsupp_strndup(const char *_str, size_t _n);
size_t         libsupp_strnlen(const char *_str, size_t _n);


/*
 *  Functions implemented in libsupp 5.0.
 */

#include <dirent.h>

#if __DJGPP_MINOR__ < 4

#define DT_REG      0x1
#define DT_BLK      0x2
#define DT_CHR      0x3
#define DT_DIR      0x4
#define DT_FIFO     0x5
#define DT_LABEL    0x6
#define DT_LNK      0x7
#define DT_UNKNOWN  0xf

#define DTTOIF(dt)  (((dt) == DT_UNKNOWN ? 0 : (dt) - 1) << 12)


# include <sys/types.h>

# undef  dirent
# define dirent     libsupp_dirent
struct dirent {
  char d_namlen;
  char d_name[256];
  mode_t d_type;
};

DIR           *libsupp_opendir_2_03(const char *_dir_name);
struct dirent *libsupp_readdir_2_03(DIR *_dirp);

int            libsupp_open_2_03(const char* _filename, int _oflag, ...);
int            libsupp_close_2_03(int _handle);
int            libsupp_fchdir_2_03(int _fd);
#else
DIR           *libsupp_opendir(const char *_dir_name);
struct dirent *libsupp_readdir(DIR *_dirp);
#endif
void           libsupp_rewinddir(DIR *_dirp);


/*
 *  Functions implemented in libsupp 5.1.
 */

int            libsupp_closedir(DIR *_dirp);


/*
 *  Functions implemented in libsupp 5.2.
 */

#include <sys/stat.h>

#if __DJGPP_MINOR__ < 4
int            libsupp_access_2_03(const char *_fn, int _flags);
int            libsupp___chdir_2_03(const char *_mydirname);
void           libsupp__fixpath_2_03(const char *_in, char *_out);
int            libsupp_stat_2_03(const char *_path, struct stat *_statbuf);
int            libsupp_symlink_2_03(const char *_source, const char *_dest);
#else
int            libsupp_access(const char *_fn, int _flags);
int            libsupp___chdir(const char *_mydirname);
void           libsupp__fixpath(const char *_in, char *_out);
char          *libsupp___canonicalize_path(const char *_in, char *_out, size_t _path_max);
int            libsupp_lstat(const char *_path, struct stat *_statbuf);
char          *libsupp_realpath(const char *_in, char *_out);
#endif


/*
 *  Functions implemented in libsupp 6.0.
 */

#include <time.h>

size_t         libsupp_strftime(char *_s, size_t _maxsize, const char *_format, const struct tm *_t);
void          *libsupp_memmem(const void *_haystack, size_t _haystack_len, const void *_needle, size_t _needle_len);

#include <wctype.h>

wint_t         libsupp_towlower(wint_t _c);

#ifndef USE_DJGPP_STRL_FCTNS
# include <stddef.h>

size_t         libsupp_strlcpy(char *_dst, const char *_src, size_t _sz);
size_t         libsupp_strlcat(char *_dst, const char *_src, size_t _sz);
#endif


/*
 *  Functions implemented in libsupp 6.1.
 */

#ifndef USE_DJGPP_MBTOWC_FCTNS
# include <wchar.h>

/*  Conversion state information.  */
typedef struct
{
  int __count;
  union
  {
    wint_t __wch;
    unsigned char __wchb[4];
  } __value;  /*  Value so far.  */
} libsupp_mbstate_t;

# undef  mbstate_t
# define mbstate_t  libsupp_mbstate_t

int            libsupp_mblen(const char *_s, size_t _n);
int            libsupp_mbsinit(const libsupp_mbstate_t *_ps);
size_t         libsupp_mbrtowc(wchar_t *_pwc, const char *_s, size_t _n, libsupp_mbstate_t *_ps);
int            libsupp_mbtowc(wchar_t *_pwc, const char *_s, size_t _n);
#endif

/*---------------------------------------------------------------------------*/


/*
 *  Make them visible.
 */

#undef  _doprnt
#define _doprnt                     libsupp__doprnt
#undef  asnprintf
#define asnprintf                   libsupp_asnprintf
#undef  asprintf
#define asprintf                    libsupp_asprintf
#undef  cprintf
#define cprintf                     libsupp_cprintf
#undef  fprintf
#define fprintf                     libsupp_fprintf
#undef  printf
#define printf                      libsupp_printf
#undef  snprintf
#define snprintf                    libsupp_snprintf
#undef  sprintf
#define sprintf                     libsupp_sprintf
#undef  vasnprintf
#define vasnprintf                  libsupp_vasnprintf
#undef  vasprintf
#define vasprintf                   libsupp_vasprintf
#undef  vfprintf
#define vfprintf                    libsupp_vfprintf
#undef  vprintf
#define vprintf                     libsupp_vprintf
#undef  vsnprintf
#define vsnprintf                   libsupp_vsnprintf
#undef  vsprintf
#define vsprintf                    libsupp_vsprintf

#undef  fseeko
#define fseeko                      libsupp_fseeko
#undef  ftello
#define ftello                      libsupp_ftello

#undef  __signbitd
#define __signbitd                  libsupp___signbitd
#undef  __signbitf
#define __signbitf                  libsupp___signbitf
#undef  __signbitld
#define __signbitld                 libsupp___signbitld

#ifndef signbit
#define signbit(x)   (__extension__ ({__typeof__(x) __x = (x); \
                                      (sizeof(__x) == sizeof(float)) ? __signbitf(__x) :  \
                                      (sizeof(__x) == sizeof(double)) ? __signbitd(__x) : \
                                      __signbitld(__x); \
                                     }))
#endif /* !signbit */

#undef  strtod
#define strtod                      libsupp_strtod
#undef  strtof
#define strtof                      libsupp_strtof
#undef  strtold
#define strtold                     libsupp_strtold
#undef  _strtold
#define _strtold                    libsupp_strtold

#undef  argz_add
#define argz_add                    libsupp_argz_add
#undef  argz_add_sep
#define argz_add_sep                libsupp_argz_add_sep
#undef  argz_append
#define argz_append                 libsupp_argz_append
#undef  argz_count
#define argz_count                  libsupp_argz_count
#undef  argz_create
#define argz_create                 libsupp_argz_create
#undef  argz_create_sep
#define argz_create_sep             libsupp_argz_create_sep
#undef  argz_delete
#define argz_delete                 libsupp_argz_delete
#undef  argz_extract
#define argz_extract                libsupp_argz_extract
#undef  argz_insert
#define argz_insert                 libsupp_argz_insert
#undef  argz_next
#define argz_next                   libsupp_argz_next
#undef  argz_replace
#define argz_replace                libsupp_argz_replace
#undef  argz_stringify
#define argz_stringify              libsupp_argz_stringify

#undef  strndup
#define strndup                     libsupp_strndup
#undef  strnlen
#define strnlen                     libsupp_strnlen

#if __DJGPP_MINOR__ < 4
# undef  opendir
# define opendir                    libsupp_opendir_2_03
# undef  readdir
# define readdir                    libsupp_readdir_2_03
# undef  open
# define open                       libsupp_open_2_03
# undef  close
# define close                      libsupp_close_2_03
# undef  fchdir
# define fchdir                     libsupp_fchdir_2_03
#else
# undef  opendir
# define opendir                    libsupp_opendir
# undef  readdir
# define readdir                    libsupp_readdir
#endif
#undef  rewinddir
#define rewinddir                   libsupp_rewinddir

#undef  closedir
#define closedir                    libsupp_closedir

#if __DJGPP_MINOR__ < 4
# undef  access
# define access                     libsupp_access_2_03
# undef  chdir
# define chdir                      libsupp___chdir_2_03
# undef  __chdir
# define __chdir                    libsupp___chdir_2_03
# undef  _fixpath
# define _fixpath                   libsupp__fixpath_2_03
# undef  stat
# define stat(file, statbuf)        libsupp_stat_2_03(file, statbuf)
# undef  symlink
# define symlink                    libsupp_symlink_2_03
# undef  access
# define access                     libsupp_access_2_03
#else
# undef  access
# define access                     libsupp_access
# undef  chdir
# define chdir                      libsupp___chdir
# undef  __chdir
# define __chdir                    libsupp___chdir
# undef  _fixpath
# define _fixpath                   libsupp__fixpath
# undef  __canonicalize_path
# define __canonicalize_path        libsupp___canonicalize_path
# undef  lstat
# define lstat                      libsupp_lstat
# undef  realpath
# define realpath                   libsupp_realpath
#endif

#ifndef USE_DJGPP_STRLCPY
# undef  strlcpy
# define strlcpy                    libsupp_strlcpy
# undef  strlcat
# define strlcat                    libsupp_strlcat
#endif
#undef  memmem
#define memmem                      libsupp_memmem
#undef  strftime
#define strftime                    libsupp_strftime
#undef  towlower
#define towlower                    libsupp_towlower

#ifndef USE_DJGPP_MBTOWC_FCTNS
# undef  mblen
# define mblen                      libsupp_mblen
# undef  mbrtowc
# define mbrtowc                    libsupp_mbrtowc
# undef  mbtowc
# define mbtowc                     libsupp_mbtowc
# undef  mbsinit
# define mbsinit                    libsupp_mbsinit
#endif
#ifdef __cplusplus
}
#endif

#endif /* !__libsupp_h_ */
